require "net/http"
require "net/https"
require "uri"

# Custom provider for fetching and decrypting password from
# ASM deployer using passed in ASM token.
Puppet::Type.type(:user).provide(
  :asm_decrypt_token,
  :parent => Puppet::Type.type(:user).defaultprovider
) do
  desc "ASM token-based password management"

  has_features :libuser, :manages_passwords

  def initialize(value={})
    super(value)
  end

  # Fetch and decrypt password if password is an ASMTOKEN string
  #
  # If the password param starts with 'ASMTOKEN-' then fetch the
  # corresponding decrypted password value and return that. Otherwise
  # return the password unaltered.
  #
  # @return [String] decrypted password from ASM token or unaltered password
  def optional_decrypt(password)
    return fetch_decrypted_password(password) if password =~ /^ASMTOKEN-/
    password
  end

  # Create user
  #
  # A workaround for an issue discovered while debugging ASM-7756.
  # The windows_adsi provider doesn't call our getter when creating
  # a user - rather it sets the value in the resource directly. 
  # This results in a newly created Windows user whose password
  # is the ASM token string rather than the decrypted password. As a 
  # workaround, we override the create method call, temporarily
  # set the (optionally) decrypted password, call the parent create
  # method and the restore the original value. 
  def create
    saved = @resource[:password]
    @resource[:password] = optional_decrypt(saved)
    super
    @resource[:password] = saved
  end

  # Get password
  #
  # Call parent class to get the password. If it is an ASM token
  # then fetch the decrypted value and return that. Otherwise
  # return the password unaltered.
  #
  # @return [String] decrypted password from ASM token or unaltered password
  def password
    optional_decrypt(super)
  end

  # Set password
  #
  # Fetch the decrypted password from ASM deployer if input is an
  # ASM token and use that value for the set operation. Otherwise
  # just use the value passed in. Call parent class to set the value.
  #
  # @param password [String] ASM token or password
  # @return [String] decrypted token password or the unaltered password
  def password=(password)
    super(optional_decrypt(password))
  end

  # Fetch decrypted password from ASM deployer
  #
  # @param token [String] token used to fetch the decrypted password
  # @return [String] decrypted password
  # @raise [StandardError] if HTTP request fails
  def fetch_decrypted_password(token)
    cert_name = Puppet.settings[:certname]
    begin
      uri = URI.parse("http://dellasm:8080/asm/secret/tokencred/%s?token_key=%s" % [cert_name, token])
      connection = Net::HTTP.new(uri.host, uri.port)
      request = Net::HTTP::Get.new(uri.to_s)
      response = connection.request(request)
    rescue Exception => e
      raise "HTTP EXCEPTION: %s" % [e.message]
    ensure
       connection.finish if connection.active?
    end
    if response.code == "200" || response.code == "202"
      pwd = response.body
    else
      raise "HTTP ERR %s:%s" % [response.code, response.message]
    end
    pwd
  end
end
