require "spec_helper"

describe Puppet::Type.type(:user).provider(:default) do

  let (:resource) {Puppet::Type.type(:user).new({:name => "test_user",
                                                 :provider => "asm_decrypt_token"})}
  let (:provider) {resource.provider}

  before (:each) do
    @connection = mock("fake_connection")
    Net::HTTP.stubs(:new).returns(@connection)

    @request = mock("fake_request");
    Net::HTTP::Get.stubs(:new).returns(@request)
    @response = mock("fake_response")
  end

  describe "for create" do

    it "should save and restore the existing resource password" do
      resource[:password] = "ASMTOKEN-xxx"

      @connection.expects(:request).with(@request).returns(@response)
      @connection.expects(:active?).returns(false)

      @response.stubs(:code).returns("200")
      @response.stubs(:body).returns("test_password")

      provider.expects(:execute).with(['/usr/sbin/useradd', '-p',
                                       'test_password', 'test_user'],
                                       {:failonfail => true, :combine =>true, 
                                        :custom_environment => {}})
      expect(provider.create)
      expect(resource[:password]).to eq("ASMTOKEN-xxx")
    end
  end

  describe "for optional_decrypt" do

    it "should fetch decrypted password if password starts with ASMTOKEN-" do
      @connection.expects(:request).with(@request).returns(@response)
      @connection.expects(:active?).returns(false)

      @response.stubs(:code).returns("200")
      @response.stubs(:body).returns("test_password")
      expect(provider.optional_decrypt("ASMTOKEN-xxx")).to eq("test_password")
    end

    it "should not fetch decrypted password if password contains (but doesn't start with) ASMTOKEN-" do
      expect(provider.optional_decrypt("xxx-ASMTOKEN-xxx")).to eq("xxx-ASMTOKEN-xxx")
    end

    it "should not fetch decrypted password if password does not start with ASMTOKEN-" do
      expect(provider.optional_decrypt("xxx")).to eq("xxx")
    end
  end

  describe "for fetch_decrypted_password" do
    
    it "should return decrypted password when HTTP request returns 200" do
      @connection.expects(:request).with(@request).returns(@response)
      @connection.expects(:active?).returns(false)

      @response.stubs(:code).returns("200")
      @response.stubs(:body).returns("test_password")
      expect(provider.fetch_decrypted_password("ASMTOKEN-dummy-uuid")).to eq("test_password")
    end

    it "should return decrypted password when HTTP request returns 202" do
      @connection.expects(:request).with(@request).returns(@response)
      @connection.expects(:active?).returns(false)

      @response.stubs(:code).returns("202")
      @response.stubs(:body).returns("test_password")
      expect(provider.fetch_decrypted_password("ASMTOKEN-dummy-uuid")).to eq("test_password")
    end

    it "should raise an error when HTTP request returns neither 200 nor 202" do
      @connection.expects(:request).with(@request).returns(@response)
      @connection.expects(:active?).returns(false)

      @response.stubs(:code).returns("404")
      @response.stubs(:body).returns(nil)
      @response.stubs(:message).returns("NOT FOUND")
      expect{provider.fetch_decrypted_password("ASMTOKEN-dummy-uuid")}.to raise_exception("HTTP ERR 404:NOT FOUND")
    end

    it "should raise an error when HTTP error is raised" do
      @connection.expects(:request).with(@request).raises("Interwebs broken")
      @connection.expects(:active?).returns(false)
      
      expect{provider.fetch_decrypted_password("ASMTOKEN-dummy-uuid")}.to raise_error("HTTP EXCEPTION: Interwebs broken")
    end
  end
end  
